/*  $Id: Hub.java 1323 2012-05-24 06:20:59Z sdevaux $
 *
 * Tabs = 4 chars
 * copyright (C) 2010 Sebastien Devaux
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 *              Sebastien Devaux <sebastien.devaux@laposte.net>
 */
package net.eduvax.util;

import java.util.concurrent.LinkedBlockingQueue;


/**
 * A hub is a connector to a fifo queue that serves severals registered handlers. 
 * @version $Id: Hub.java 1323 2012-05-24 06:20:59Z sdevaux $
 * @author Sebastien Devaux
 */
public class Hub<T> implements Runnable, Handler<T> {
	/**
	 * create new hub
	 * @param name hub name
	 */
	public Hub(String name) {
		_name=name;
        _handlers=new HandlerList<T>();
        _fifo=new LinkedBlockingQueue<T>();
	}
	/**
	 * Start hub with its own thread
	 */
	public void start() {
		Thread th=new Thread(this,""+getClass()+"-"+_name);
		th.start();
	}
	/**
	 * Handling object by hub is enqueuing it.
	 * @param obj object to enqueue
	 */
	public void handle(T obj) {
		_fifo.add(obj);
	}
	/**
	 * Defines obj to use as stop signal when found in queue.
	 * Stop object is transmited to all register handlers. All object remaining
	 * in queue are ignored unless the hub is restarted.
	 */
	public void setStopObject(T stop) {
		_stopObject=stop;
	}
	/** 
	 * register handler
	 * @param handler handler to register to this hub
	 */
	public void add(Handler<T> handler) {
		_handlers.add(handler);
	}
	/** 
	 * deregister handler
	 * @param handler handler to remove from this hub
	 */
	public void remove(Handler<T> handler) {
		_handlers.remove(handler);
	}
	/**
	 * Request Hub shutdown.
	 * Shutdown may occure lately. Methods returns immediately, but stop really occurs
	 * when all object already inserted are handled.
	 */
	public void stop() {
		handle(_stopObject); 
	}
	// runnable implementation.
	public void run() {
		T obj;
        try {
            do {
                obj=_fifo.take();
                _handlers.handle(obj);
            } 
            while (obj!=null && obj!=_stopObject);
        }
        catch (InterruptedException ex) {
            App.get().log().warn("Hub "+_name+" interrupted: "+ex);
        }
	}
	/**
	 * HandlerList containing hub member handlers
	 * @djin.relation composition
	 */
	private HandlerList<T> _handlers;
	/**
	 * Event queue, as far as T type is an event. 
	 * So may be an "anything to handle" queue.
	 * @djin.relation aggregation,-,*,T
	 */
	private LinkedBlockingQueue<T> _fifo;
	/** hub name */
	private String _name;
	/** object to insert to order Hub shutdown */
	private T _stopObject=null;
}
